"""Effective Ajax in TurboGears

Presentation by Kevin Dangoor
dangoor+eajax@gmail.com

Copyright 2006 Kevin Dangoor.
"""

import sys
from turbogears import expose, widgets, controllers, mochikit, validate, \
                       flash, redirect
from turbogears.validators import JSONValidator
import pkg_resources
from effectiveajaxturbogears import autocomp, rtu, snippets
from snippets import *

# set up for the JavaScript we need
js_dir = pkg_resources.resource_filename("effectiveajaxturbogears",
                                        "static/javascript")
widgets.register_static_directory("ajax", js_dir)
proto = widgets.JSLink("ajax", "prototype.js")
scriptaculous_js = widgets.JSLink("ajax", "scriptaculous.js")
class Scriptaculous(widgets.Widget):
    javascript = [proto, scriptaculous_js, mochikit]

scriptaculous = Scriptaculous()

shcss = widgets.CSSLink("ajax", "sh/SyntaxHighlighter.css")
shcore = widgets.JSLink("ajax", "sh/shCore.js")
shpython = widgets.JSLink("ajax", "sh/shBrushPython.js")
shxml = widgets.JSLink("ajax", "sh/shBrushXml.js")
shjscript = widgets.JSLink("ajax", "sh/shBrushJScript.js")
shhighlight = widgets.JSSource(
            "dp.SyntaxHighlighter.HighlightAll('code');",
            location=widgets.js_location.bodybottom)

class SyntaxHighlighter(widgets.Widget):
    css = [shcss]
    javascript = [shcore, shpython, shxml, shjscript, shhighlight]

highlighter = SyntaxHighlighter()

# Here are the presentation elements

slides = [
    ["title", "Effective Ajax in TurboGears"],
    ["hi", '<img src="/static/images/HiMyNameIs.jpg"/>'],
    ["normal", '<img src="/static/images/NormalPeople.jpg"/>'],
    ["benefit", "Benefits &#8212; Not Buzzwords"],
    ["what-is-ajax", "Ajax is Asynchronous JavaScript and XML"],
    ["what-is-ajax2", "Ajax is Asynchronous JavaScript and <strike>XML</strike>"],
    ["what-is-ajaj", "Asynchronous JavaScript and JSON"],
    ["raisedbar", "Do I need Ajax?"],
    ["yahoonews", '<img src="/static/images/YahooNews.png"/>'],
    ["canyon-of-despair", """<h1 align="center">The IE5/NS4 Canyon of Despair</h1>
<p align="center"><img src="/static/images/FlamingCanyon.jpg"/></p>"""],
    ["almost-there", '<img src="/static/images/IEFFS.jpg"/>'],
    ["jstoolkits", '<img src="/static/images/DojoMochi.jpg"/>'],
    ["improving", ""],
    ["text-entry", '<img src="/static/images/TextEntry.jpg"/>'],
    ["symbols/", ""],
    ["beenthere-donethat", "People know this kind of control"],
    ["symbolsindex", symbolsindex],
    ["symbolsbody", symbolsbody],
    ["symbolsearch", symbolsearch],
    ["symbolsjs", symbolsjs],
    ["symbolsget", symbolsget],
    ["symbolend", ""],
    ["realtime-updates", "Real Time Updates"],
    ["jobs", '<img src="/static/images/stevejobs.jpg"/>'],
    ["docupdate/", "Document update example"],
    ["rtumodel", rtumodel],
    ["rtuform", rtuform],
    ["rtueditcode", rtueditcode],
    ["rtuedittemplate", rtuedittemplate],
    ["rtueditjs", rtueditjs],
    ["rtucheckchange", rtucheckchange],
    ["ordered", "Ordered lists"],
    ["myyahoo", '<img align="center" src="/static/images/MyYahooMoveItem.png"/>'],
    ["orderedexample", "Example"],
    ["cookiedisplay", cookiedisplay],
    ["cookiejs", cookiejs],
    ["cookiecode", cookiecode],
    ["orderedend", ""],
    ["the-confusers", "The Confusers"],
    ["like-browsers", "People like their browsers"],
    ["back-button", "Hey! what happened to my back button"],
    ["dojo-back", "Dojo has a solution"],
    ["dont-use-ajax", "Or don't use Ajax"],
    ["bookmarks", "Why can't I bookmark this?"],
    ["dojo-bookmarks", "Dojo can"],
    ["google-maps", '<img align="center" src="/static/images/GmapsLink.png"/>'],
    ["not-how-control", "That's not how this control should work!"],
    ["controlexample", "example"],
    ["votehtml", votehtml],
    ["votejs", votejs],
    ["votecontroller", votecontroller],
    ["voteend", ""],
    ["depends-on-audience", "Know your users"],
    ["build-on-experience", "Build on what they know"],
    ["expectations", "Don't break what they expect to work"],
    ["not-a-silver-bullet", '<img src="/static/images/SilverBullet.jpg"/>'],
]

cookies = ["Thin Mints", "Peanut Butter Patties", "Samoas", "Do-si-dos",
           "Caramel deLites", "Tagalongs"]

your_vote = None

class Root(controllers.RootController):
    symbols = autocomp.AutoCompleteExample()
    docupdate = rtu.RealTimeUpdateExample()
    
    @expose()
    def default(self, page):
        prev = None
        next = None
        for i in range(0, len(slides)):
            if slides[i][0] == page:
                break
            prev = slides[i][0]
        if i+1 < len(slides):
            next = slides[i+1][0]
        else:
            next = slides[0][0]
        the_words = str(slides[i][1])
        if the_words.startswith("<") and not the_words.startswith("<img"):
            additional = the_words
            the_words = None
        else:
            additional = None
        if ".kid" == the_words:
            tg_template = "effectiveajaxturbogears.templates.pages.%s" \
                % (page)
        else:
            tg_template="effectiveajaxturbogears.templates.generic"
        return dict(tg_template=tg_template,
            the_words=the_words, additional=additional,
            next=next, prev=prev, sh=highlighter)
    
    @expose(template=".templates.welcome")
    def index(self):
        return dict(no_nav=True)
    
    @expose(template=".templates.paper")
    def paper(self):
        return dict(no_nav=True, sh=highlighter, snippets=snippets)
    
    @expose(template=".templates.snippets")
    def snippets(self):
        examples = []
        snippets = []
        last_example = None
        for slide in slides:
            snip = slide[1]
            if isinstance(snip, formatted):
                if snip.example != last_example:
                    if snippets:
                        examples.append((last_example, snippets))
                    snippets = []
                    last_example = snip.example
                snippets.append(snip)
        if snippets:
            examples.append((last_example, snippets))
        return dict(examples = examples)
    
    @expose(template=".templates.ordered")
    def orderedexample(self):
        return dict(cookies = cookies, prev="notajax",
            next="cookiedisplay", scripty=scriptaculous)
    
    @expose(format="json")
    @validate(validators={"cookielist" : JSONValidator()})
    def saveorder(self, cookielist):
        global cookies
        cookies = cookielist
        return dict()
    
    @expose(template=".templates.controlexample")
    def controlexample(self):
        return dict(scripty=scriptaculous, prev="not-how-control",
            next="votehtml")
    
    @expose(format="json")
    def savevote(self, vote):
        global your_vote
        if your_vote:
            return dict(already=True, voted=your_vote)
        else:
            your_vote = vote
            return dict(already=False, voted=vote)