import time
from turbogears import expose, widgets, controllers, mochikit, validate, \
                       flash, redirect
from turbogears.validators import Int, Number

class PersonPaid(object):
    def __init__(self, id, name, paid):
        self.name = name
        self.paid = paid
        self.id = id
        
    def __setattr__(self, attr, value):
        super(PersonPaid, self).__setattr__(attr, value)
        super(PersonPaid, self).__setattr__("last_update", time.time())

poormansdb = {
    1: PersonPaid(1, "Gandalf", "paid"),
    2: PersonPaid(2, "Albus Dumbledore", "dog"),
    3: PersonPaid(3, "Merlin", "deadbeat"),
    4: PersonPaid(4, "Raistlin Majere", "shortchanged")
}

paidoptions = dict(
    paid="Paid in Full", 
    shortchanged="Shortchanged Us",
    dog="Dog Ate the Bill", 
    deadbeat="Deadbeat")

class PaidStatusWidgets(widgets.WidgetsDeclaration):
    name = widgets.TextField()
    paid = widgets.RadioButtonList(options=paidoptions.items())
    id = widgets.HiddenField(validator=Int())
    last_update = widgets.HiddenField(validator=Number())

paymentform = widgets.TableForm(fields=PaidStatusWidgets())

class RealTimeUpdateExample(controllers.Controller):
    @expose(template="effectiveajaxturbogears.templates.paiddb")
    def index(self):
        return dict(db=poormansdb, paidoptions=paidoptions,
            next="rtumodel", prev="jobs")
    
    @expose(template="effectiveajaxturbogears.templates.paidedit")
    @validate(validators=dict(id=Int()))
    def paidedit(self, id):
        return dict(person=poormansdb[id], form=paymentform,
            mochi=mochikit,
            next="rtumodel", prev="jobs")
    
    @expose(format="json")
    @validate(validators=dict(id=Int(), last_update=Number()))
    def checkchanged(self, id, last_update):
        person = poormansdb[id]
        has_changed = int(person.last_update) != int(last_update)
        return dict(changed=has_changed)
    
    @expose()
    @validate(form=paymentform)
    def saveperson(self, id, name, paid, last_update):
        person = poormansdb[id]
        person.name = name
        person.paid = paid
        flash("%s status now changed to %s" % (name, paid))
        redirect("./")