class formatted:
    def __init__(self, lang, example, title, thecode):
        self.lang = lang
        self.example = example
        self.title = title
        self.thecode = thecode
        
    def __str__(self):
        return """<textarea name="code" class="%s"><![CDATA[%s]]></textarea>""" % (self.lang, self.thecode)

symbolsbody = formatted("html", "Symbol Search", "Template Body", 
"""<body bgcolor="white">
    <h1 class="exampleheader">Python Symbol Search</h1>
    <p>There are ${len(all_symbols)} symbols.</p>
    ${searchbox.display()}
    <div>
      <input type="button" onclick="get_symbol()" 
        value="Show Value"/>
    </div>
    <pre id="value"></pre>
</body>
""")

symbolsearch = formatted("py", "Symbol Search", "Ajax Search Server Side",
"""@expose(format="json")
def symsearch(self, search_for):
    if len(search_for) < 3:
        return dict(symbols=[])
    search_for = search_for.lower()
    matches = ["%s.%s" % (item.mod, item.name) 
                for item in all_symbols 
                if search_for in item.name.lower()]
    return dict(symbols=matches)
""")

symbolsget = formatted("py", "Symbol Search", "Ajax Get Value Server Side",
"""@expose(format="json")
def getsym(self, symbol):
    ldot = symbol.rfind(".")
    mod = symbol[:ldot]
    var = symbol[ldot+1:]
    value = getattr(sys.modules[mod], var, None)
    return dict(value=str(value))
""")

symbolsindex = formatted("py", "Symbol Search", "Index Method",
"""class AutoCompleteExample(controllers.Controller):
    @expose(template="effectiveajaxturbogears.templates.symbols")
    def index(self):
        searchbox = widgets.AutoCompleteField(
            name="symbol",
            search_controller="symsearch", 
            search_param="search_for",
            result_name="symbols")
        return dict(searchbox=searchbox, mochi=mochikit,
                    all_symbols=all_symbols,
                    prev="text-entry", next="beenthere-donethat")
""")

symbolsjs = formatted("js", "Symbol Search", "Ajax Get Value Client Side",
"""function get_symbol() {
  d = loadJSONDoc("getsym?" + 
      queryString(
          {"symbol" : 
              $("autoCompleteform_symbol").value}));
  d.addCallback(show_value);
}

function show_value(result) {
  replaceChildNodes("value", result["value"]);
}
""")

rtumodel = formatted("py", "Real Time Update", "The 'Database'",
"""class PersonPaid(object):
    def __init__(self, id, name, paid):
        self.name = name
        self.paid = paid
        self.id = id
        
    def __setattr__(self, attr, value):
        super(PersonPaid, self).__setattr__(attr, value)
        super(PersonPaid, self).__setattr__("last_update", 
                                            time.time())

poormansdb = {
    1: PersonPaid(1, "Gandalf", "paid"),
    2: PersonPaid(2, "Albus Dumbledore", "dog"),
    3: PersonPaid(3, "Merlin", "deadbeat"),
    4: PersonPaid(4, "Raistlin Majere", "shortchanged")
}
""")

rtuform = formatted("py", "Real Time Update", "Entry Form",
"""paidoptions = dict(
    paid="Paid in Full", 
    shortchanged="Shortchanged Us",
    dog="Dog Ate the Bill", 
    deadbeat="Deadbeat")

class PaidStatusWidgets(widgets.WidgetsDeclaration):
    name = widgets.TextField()
    paid = widgets.RadioButtonList(options=paidoptions.items())
    id = widgets.HiddenField(validator=Int())
    last_update = widgets.HiddenField(validator=Number())

paymentform = widgets.TableForm(fields=PaidStatusWidgets())
""")

rtueditcode = formatted("py", "Real Time Update", "Edit Method", """@expose(template="effectiveajaxturbogears.templates.paidedit")
@validate(validators=dict(id=Int()))
def paidedit(self, id):
    return dict(person=poormansdb[id], form=paymentform,
        mochi=mochikit,
        next="rtumodel", prev="jobs")
""")

rtuedittemplate = formatted("html", "Real Time Update", "Edit Template",
"""${form.display(person, action="saveperson")}
<div id="changemsg" class="statusmsg" style="display:none">
    Data has been changed. 
    <a href="javascript:window.location.href=window.location">
      Click to refresh</a>.
</div>

""")

rtueditjs = formatted("js", "Real Time Update", "The JavaScript",
"""function checkchanged() {
  d = loadJSONDoc("checkchanged?" + queryString({
    "id" : ${person.id},
    "last_update" : ${person.last_update}
  }));
  d.addCallback(displaychange);
}

function displaychange(result) {
  if (result["changed"]) {
    showElement("changemsg");
  } else {
    window.setTimeout("checkchanged()", 1000);
  }
}

window.setTimeout("checkchanged()", 1000);
""")

rtucheckchange = formatted("py", "Real Time Update", "Update Check Server Side",
"""@expose(format="json")
@validate(validators=dict(id=Int(), last_update=Number()))
def checkchanged(self, id, last_update):
    person = poormansdb[id]
    has_changed = int(person.last_update) != int(last_update)
    return dict(changed=has_changed)
""")

cookiedisplay = formatted("html", "Cookies", "Cookie List Template",
"""<h1 class="exampleheader">Girl Scout Cookie Preference</h1>
<p>Please rank your preference of Girl Scout cookie 
varieties:</p>
<ul id="cookielist">
  <li py:for="cookie in cookies" py:content="cookie"
   id="c_${cookie}">Cookie Name Here</li>
</ul>
<script type="text/javascript">
  Sortable.create('cookielist', 
    { "onUpdate": update_server });
</script>
<div id="saved" class="statusmsg">Changes saved!</div>
""")

cookiecode = formatted("py", "Cookies", "Server Side Save Code",
"""@expose(format="json")
@validate(validators={"cookielist" : JSONValidator()})
def saveorder(self, cookielist):
    global cookies
    cookies = cookielist
    return dict()
""")

cookiejs = formatted("js", "Cookies", "The JavaScript",
"""function update_server() {
  var children = $("cookielist").childNodes;
  var cookies = new Array(children.length);
  for (var i = 0; i < children.length; i++) {
      cookies[i] = children[i].firstChild.nodeValue;
  }
  jsoncookies = serializeJSON(cookies);
  d = loadJSONDoc("saveorder?" 
      + queryString({cookielist : jsoncookies}));
  d.addCallback(update_done);
}

function update_done() {
  showElement("saved");
  window.setTimeout(function() {
      new Effect.SwitchOff("saved", {queue: "end"})},
      1000);

}
""")

votehtml = formatted("html", "Vote", "A Part of the Template",
"""<td valign="top" id="EQS">Edward Q. Scissorhands</td>
<td><input type="radio" name="vote" value="EQS"
 onclick="cast_vote(this)"/>""")

votejs = formatted("js", "Vote", "Casting the Vote with JavaScript",
"""function cast_vote(vote) {
    d = loadJSONDoc("savevote?vote=" + vote.value);
    d.addCallback(voteresponse);
}
 
function voteresponse(result) {
    if (result["already"]) {
        buttons = document.controlexample.vote;
        votevalue = result["voted"];
        for (var i = 0; i < buttons.length; i++) {
            if (buttons[i].value == votevalue) {
                buttons[i].checked = true;
                break;
            }
        }
        showElement("already");
        window.setTimeout(function() {
            new Effect.Fade("already", {queue: "end"})}, 
            3500);
    } else {
        new Effect.Highlight(result["voted"]);
    }
}
""")

votecontroller = formatted("py", "Vote", "Server Side Vote Casting",
"""@expose(format="json")
def savevote(self, vote):
    global your_vote
    if your_vote:
        return dict(already=True, voted=your_vote)
    else:
        your_vote = vote
        return dict(already=False, voted=vote)
""")