<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
    py:extends="'master.kid'">

<head>
    <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
    <title>Effective Ajax with TurboGears</title>
    <style type="text/css">
        .dp-highlighter {
            clear: both;
        }
        .dp-highlighter td 
        {
        	font-size: 14px;
        	line-height: 16px;
        }
    </style>
</head>

<body bgcolor="white">

<h1 id="effective_ajax_with_turbogears" class="exampleheader">Effective Ajax With TurboGears</h1>

<p>Kevin Dangoor, Blazing Things LLC</p>

<p>February 26, 2006</p>

<p>(Note that this should not be confused with [Rob Sanheim's][http://www.robsanheim.com/] Effective Ajax presentation.)</p>

<h2 id="introduction">Introduction</h2>

<p>It's been a year since Jesse James Garrett of Adaptive Path coined the term <a href="http://www.adaptivepath.com/publications/essays/archives/000385.php">"Ajax"</a> and nearly two years since Google showed everyone that browsers could be a lot richer than most people were taking advantage of (with Gmail). Ajax, for those not following this sort of thing, stands for Asynchronous JavaScript + XML. Ajax is all about providing a more interactive web application by having the browser request additional information without a page refresh. XML is not always used, as we'll see here, but you can't escape JavaScript and it'll definitely be asynchronous if you don't want to lock up the user's browser.</p>

<p>Speaking of things that you don't want to do to your users, that's what "Effective Ajax with TurboGears" is all about. Ajax is still new territory for many, and it's a wild frontier of experimentation. It seems like every day that there's a ne.w.si.te showing off some technique or other that makes your browser jump into the 21st century. Experiments are fine, but much of our time is spent making applications that are used by normal people who are just trying to get their jobs done.</p>

<p>This article explores some thoughts, with examples, of how you can effectively use Ajax today to provide a better user experience. In the brief space available, this article is going to completely ignore the large and important issue of accessibility. There are real concerns with Ajax and accessibility, and that is a separate topic.</p>

<h2 id="current_user_expectations">Current User Expectations</h2>

<p>People have been experimenting with Ajax-like techniques for years. I remember coming across Brent Ashley's <a href="http://www.blueshoes.org/en/javascript/jsrs/">JavaScript Remote Scripting library</a> in 2001. But, Gmail was the application that brought the idea to the forefront and Google Maps showed that the idea was not a one-trick pony. Google developers were certainly not the only ones using this technique, but they made the biggest splash. The Adaptive Path article came at just the right time to give the technique a name, just as developers all over were starting to embrace the technique.</p>

<p>Depending on your target audience, it's hard to ignore Ajax for many applications these days. Even something as low-tech as reading a news story on Yahoo! has become Ajaxified, highlighting key terms and running searches in-page for you.</p>

<p><img src="/static/images/YahooNews.png" alt="Yahoo! News screenshot" title="" /></p>

<p>So, the bar has been raised. Users want Ajax, even if they're not going to ask for it by name. More specifically, though, users want <em>good</em> use of Ajax.</p>

<h2 id="state_of_the_browser">State of the Browser</h2>

<p>If we were still living in an Internet Explorer 5/Netscape 4 world, I think that web developers would be more inclined to jump into a flaming, alligator-filled canyon than write something like Gmail.</p>

<p><img src="/static/images/FlamingCanyon.jpg" style="float: left; padding-right: 1em" alt="Picture of a flaming, alligator-filled canyon" title="" /></p>

<p>Luckily, IE6, Firefox and Safari together add up to <a href="http://marketshare.hitslink.com/report.aspx?qprid=6">nearly 95% of the browsers</a> in use today. These three browsers are far more standards-compliant than their predecessors, which means that there's now a non-zero chance that code you write for one might work vaguely similarly on the others. Actually, it's a bit better than that. Many things look and work the same between those three browsers, and diligent folks working on projects like <a href="http://dojotoolkit.com">Dojo</a> and <a href="http://mochikit.com">MochiKit</a> have encapsulated many of the differences for you.</p>

<p>The state of the browser is much, much better than it used to be. If you happen to be in a large corporate environment where IT has mandated that everyone must run Netscape 4.7, the canyon is right over there to your left.</p>

<h2 id="enhancer_or_confuser">Enhancer or Confuser</h2>

<p>There are certain uses of Ajax that are pure wins. They enhance the user experience and are intuitively obvious to people, based on their prior experiences.</p>

<p>But, the use of Ajax is still new and things that may seem easy to you may be confusing to others. If something is a little confusing but can be a big boon to productivity, you might want to use that approach, particularly if your users are people that can be trained. A site like Amazon.com that needs to be easy for everyone will avoid confusion as much as possible. How do they know what confuses people? Amazon.com tests out their ideas to see what effect they have on sales.</p>

<p>Then there are things that might seem really cool, but they just confuse people without providing them with a productivity boost. Do those things if you're tired of running a profitable business or being employed.</p>

<h1 id="ajax_as_an_enhancer" class="exampleheader">Ajax as an Enhancer</h1>

<p>Some Ajax is a pure win. Do more of that and people will love you for it. (Please think of me at your IPO.)</p>

<h2 id="improving_text_entry">Improving Text Entry</h2>

<p>One of Google's many perennial betas is <a href="http://www.google.com/webhp?complete=1&amp;hl=en">Google Suggest</a>. As you type into the text box, a drop-down list of search terms appears, letting you choose from the menu. Suggest even lists the number of matches for each search term, which helps give you an idea if one set of terms may be better than another for the search you're doing.</p>

<p>This kind of feature is definitely a user-interface enhancer. It provides the user with a convenient pick list, even among large numbers of entries. It reduces the amount of typing the user needs to do. It also reduces the amount of thinking they need to do, by avoiding possible misspellings.</p>

<p>And, importantly, this kind of autocompletion is something that users are <em>already used to</em>. This is a vitally important thing to keep in mind when you're working on a spiffy new Ajax control: if you start doing things that your users aren't used to, you've got a big education battle ahead of you. And you'll probably lose. Desktop applications already have a number of useful controls, metaphors and ideas that you can bring across to your webapps. Google Suggest's autocompletion is something that Quicken, for one, has had for years.</p>

<p>Of course, even when dealing with something that people are familiar with, you still need to pay attention to <em>how</em> you implement the feature. Thomas Baekdal points out that <a href="http://www.baekdal.com/articles/Usability/XMLHttpRequest-guidelines">a live search can distract people</a> from what they're trying to do.</p>

<h2 id="autocompletion_example">Autocompletion Example</h2>

<p><img src="/static/images/SymbolComplete.png" style="float:right"/></p>

    <p>The software running the visuals for the Effective Ajax with TurboGears presentation was itself written in TurboGears. The autocompletion example performs simple searches of all of the names in the Python namespace. As of this writing, that is about 13,300 names. That's far too many for a pick list, but it works great with autocompletion.</p>
    
    <p>In addition to using Ajax to perform autocompletion among those 13,000 names, this demo will also do an Ajax request to look up the stringified value of the object you pick.</p>
    
    <p>Let's take a look at how the autocompletion is set up:</p>
    
    <div py:replace="XML(str(snippets.symbolsindex))"/>
    
    <p>The important thing to note in this example is that we're using an AutoCompleteField widget. This type of widget is commonplace enough and important enough that TurboGears includes it out of the box. To use it, we need to do just a little bit of configuration. <code>search_controller</code> provides the URL (a relative URL in this case) that will perform the search for the widget. <code>search_param</code> specifies the name of the parameter that needs to be passed to the <code>search_controller</code>. <code>result_name</code> tells the widget what name to look up in the resulting hash table in order to find the list of matches.</p>
    
    <p>The dictionary returned by the method will get passed to the template. By including the AutoCompleteField widget and the mochikit widget, the required JavaScript will automatically be included in the final pages.</p>
    
    <p>Note that the "next" and "prev" values returned in the dictionary are used for navigation through the presentation visuals. Those have no function for the demo itself.</p>
    
    <p>Next, we'll take a look at the template body that generates the page for this demo:</p>
    
    <div py:replace="XML(str(snippets.symbolsbody))"/>
    
    <p>In this template snippet, we see <code>$${searchbox.display()}</code>. That is all it takes to display the text field and the little spinner image next to it. After the autocomplete widget, we have the "Show Value" button, which calls a JavaScript function called <code>get_symbol</code>, which we'll look at shortly. We include an empty &lt;pre&gt; tag that we fill in later with the value of the Python symbol we're looking up.</p>
    
    <div py:replace="XML(str(snippets.symbolsearch))"/>
    
    <p>Above, we see the Python controller method that performs the search on behalf of the AutoCompleteField widget. We had told the AutoCompleteField that its <code>search_controller</code> is called <code>symsearch</code>, and you'll note that that is the name of this method. At the top, you see that the format provided for this method is "json". The dictionary we return at the end of the method is automatically turned into a JSON string. As we had specified when we created the widget, the parameter with the search string is called <code>search_for</code> and the result we provide in the JSON output is named "symbols". (Clearly, the "run straight through the entire list" search algorithm being used is not the most efficient. That's not the point of the demo app, though.)</p>
    
    <p>The AutoCompleteField widget includes all of the JavaScript it needs to work its magic. Let's take a look at the JavaScript used to retrieve the value of the symbol:</p>
    
    <div py:replace="XML(str(snippets.symbolsjs))"/>
    
    <p><code>get_symbol</code> uses MochiKit's <code>loadJSONDoc</code> function to call a method called <code>getsym</code> on the server. It looks up the value of the AutoCompleteField widget we created and passes that string over in a parameter called <code>symbol</code>. When the result comes back (asynchronously), MochiKit will call the <code>show_value</code> function.</p>
    
    <p><code>show_value</code> is very simple. It just uses MochiKit's <code>replaceChildNodes</code> function to fill in our empty &lt;pre&gt; tag with the stringified value of the Python object. MochiKit's <code>loadJSONDoc</code> function handles the conversion of the JSON string that arrives in the browser into JavaScript objects.</p>
    
    <p>Finally, we see that the <code>getsym</code> method on the server side is very simple:</p>
    
    <div py:replace="XML(str(snippets.symbolsget))"/>

<h2 id="real_time_display_updates">Real Time Display Updates</h2>

<p><img src="/static/images/stevejobs.jpg" style="float:left; height: 205px; width: 352px; padding-right: 1em; padding-bottom: 1em"/></p>

<p>The web was originally an information delivery vehicle and not an application platform. Though we think of Ajax in terms of bridging the gap between webapps and desktop apps, Ajax can also help in information delivery. The web today displays quite a bit of rapidly changing information: stock quotes, election results, news headlines, and <a href="http://www.macrumorslive.com/about/">Steve Jobs keynote addresses</a> are just a few examples. By making requests to the server periodically, you can make portions of your page stay up to date while your reader is viewing a single item.</p>

<p>This technique also works for webapps. Imagine a document review application. With the proper infrastructure, you can notify the user that the document has changed while they're viewing it.</p>

<p>This technique is not without peril, however. For a real-world implementation, you need to carefully orchestrate these updates to not swamp your server and to not go over IE's 2 connection limit. Dojo's Alex Russell has <a href="http://alex.dojotoolkit.org/?p=538">written about the challenges</a> of implementing low-latency update code that works within the boundaries of what a browser can do. For a departmental application behind a firewall, odds are that these issues don't matter. If you're planning to compete with Google, you'll probably want to give it some serious thought.</p>

<p>It's also important to make sure that you don't have so many parts of your page updating in different ways and at different times that the user is unable to focus on their primary task at hand.</p>

<h2>The Document Update Example</h2>

<p><img style="float:right" src="/static/images/DocUpdate.png"/></p>

<p>For this example, imagine a database in which it's possible that two users could edit the same record at the same time. The decision is made to inform the user when this happens and give them a chance to reload the data fresh from the server.</p>

<p>In order to allow this demo to run on a stock TurboGears install (no database required), the database used for this simple demo is a Python dictionary. The data looks like this:</p>

<div py:replace="XML(str(snippets.rtumodel))"/>

<p>There's nothing too exciting happening there. Let's take a look at the form that we use for the data entry:</p>

<div py:replace="XML(str(snippets.rtuform))"/>

<p>This form consists entirely of standard HTML form control widgets. There is no JavaScript involved in this. Here is the controller code that displays the edit form:</p>

<div py:replace="XML(str(snippets.rtueditcode))"/>

<p>The interesting thing that we see here is a validator. The <code>id</code> value that is passed in by the browser is converted into an int on the way in.</p>

<div py:replace="XML(str(snippets.rtuedittemplate))"/>

<p>Above, we see the body of the editing template. We start by displaying the edit form, using the <code>person</code> value passed in by the edit method. We also tell the form where it will be posting to (<code>saveperson</code>).</p>

<p>The change message that appears below the form is not displayed until the JavaScript on the page detects the page. The JavaScript holds the key to the update example:</p>

<div py:replace="XML(str(snippets.rtueditjs))"/>

<p>At the bottom of the JavaScript, we see a timeout set. We tell the browser to call the <code>checkchanged</code> function after one second. <code>checkchanged</code> uses <code>loadJSONDoc</code> to call the server to find out if the data has changed, passing along the <code>id</code> and <code>last_update</code> time for that person. The server will determine if the data has changed and, when it does, the JavaScript <code>displaychange</code> function is called.</p>

<p><code>displaychange</code> just looks at the result to see whether the data has changed. If it has, we show our change message and no longer check for changes. If it hasn't we tell the browser to check again in a second.</p>

<p>Clearly, doing a check for new data every second can get expensive. There are different ways to request updated data from the server (some of which keep the connection to the server open persistently to make latency as low as possible). This type of feature is something you'll definitely need to tune for your own application and deployment requirements.</p>

<p>The <code>checkchanged</code> method on the server is very similar to others that we've seen:</p>

<div py:replace="XML(str(snippets.rtucheckchange))"/>

<h2 id="ordered_lists">Ordered Lists</h2>

<p>Reorganizing a list of items is a lot easier with drag-and-drop than with the old "click until you're in the right place" method.</p>

<p><img src="/static/images/MyYahooMoveItem.png" alt="My Yahoo! customization screen" title="" /></p>

<p>Nowadays, you can drag and drop:</p>

<p><img src="/static/images/GoogleDND.png"/></p>

<p>But, wait! This isn't Ajax! Actually, in the case of Google's personalized page it <em>is</em> Ajax. Google, correctly in my opinion, decided that there was no need for some additional confirmation once you had dragged the item to the place you wanted it. How annoying would Microsoft Word be if you had to click "Confirm" every time you moved an image on the page?</p>

<p>To Yahoo!'s credit, My Yahoo! <em>does</em> support drag and drop placement of items. Arguably, Yahoo!'s implementation is better because it gracefully supports older browsers and browsers without JavaScript enabled in addition to doing drag and drop for recent browsers.</p>

<h2>Cookie Preference Example</h2>

<p><img src="/static/images/CookieDrop.png" style="float:left"/></p>

<p>Our example for Ajaxified drag and drop is a list of Girl Scout cookie preferences. It looks like a simple unordered list, but you can drag and drop the items from one position to another. When you do so, a "Changes Saved" message will appear after the data has been saved on the server.</p>

<p>This example uses Scriptaculous. MochiKit 1.3 will include a port of Scriptaculous' features, but that wasn't quite ready at the time of this writing.</p>

<p>The template shows just how much functionality is included in the JavaScript libraries we have today. The list of cookie names is created as a standard unordered list. After we create the list, we call <code>Sortable.create</code>, passing in the ID of our list and the instruction to call <code>update_server</code> whenever the user completes a change to the list. That one call is all it takes to add drag and drop to our list!</p>

<div py:replace="XML(str(snippets.cookiedisplay))"/>

<p>Let's look at the JavaScript for <code>update_server</code> that is responsible for the Ajax part of this demo:</p>

<div py:replace="XML(str(snippets.cookiejs))"/>

<p>The first part of <code>update_server</code> goes through the DOM and picks up the value of each element in our list. It's essentially turning the list displayed onscreen into a JavaScript array. That array is then handed off to MochiKit's <code>serializeJSON</code> function, which turns it into a JSON string. We then use the familiar <code>loadJSONDoc</code> call to save the new order on the server.</p>

<p>Once the save is complete, we display the "Changes Saved" message to the user.</p>

<p>There's one interesting bit to the server side of this save operation. TurboGears includes a JSONValidator that will convert an incoming JSON string into proper Python objects. That allows us to just set the server's cookie list directly to the incoming value. Here's the code for that:</p>

<div py:replace="XML(str(snippets.cookiecode))"/>

<h1 id="ajax_as_a_confuser" class="exampleheader">Ajax as a Confuser</h1>

<p>The potential for Ajax abuse is staggering. Remember when everyone started making their own homepages with sickening backgrounds, color cycling rainbow borders and whatever dancing hamster-like web tchotzkes they could get their hands on? If we're not careful, we'll enter into the Ajax equivalent of that. Imagine dancing hamsters that change their dance depending on the state of the stock market. That's the kind of horror I'm imagining.</p>

<p>In your webapps, you're not likely to get into something quite that bad. But, it's pretty easy to do things that will make your users unhappy.</p>

<h2 id="hey_what_happened_to_my_back_button">Hey! What happened to my back button?</h2>

<p>Most people don't like having to learn a new piece of software. After several years of web browsing, people are comfortable with the controls in a web browser. I was working on a desktop application where we had the full might of a GUI toolkit at our disposal. What our users wanted, though, was a back button, just like the one in their browsers. People like their back button.</p>

<p>The whole point of Ajax is to be able to update the page without changing the user's perspective in their browser. For real-time data display or "application-like" things, this may be fine (ask your users!). But, sometimes you really want the back button to work the way people expect it to.</p>

<p>Dojo Toolkit, for one, has explicitly baked in hooks and helper code <a href="http://dojotoolkit.org/intro_to_dojo_io.html">to make the back button work as expected</a>. The kind of pain that they went through to make that work is something that you likely don't want to try to do under your project's deadlines. If you need Ajax and the back button, use Dojo or a toolkit with similar capability.</p>

<p>For some cases, there's another solution: don't use Ajax. Just because you <em>can</em> roundtrip to the server without updating the page doesn't mean that you have to or you should. If Ajax isn't actually adding any value to your app, don't use it. Avoiding it will let your user's browser work like they expect a web browser to work.</p>

<h2 id="why_cant_i_bookmark_this">Why can't I bookmark this?</h2>

<p>Closely related to the back button problem is the bookmarks problem. The same ideas apply. For Google Maps, their solution was to put a special link on the page that you could copy and paste. Dojo also <a href="http://dojotoolkit.org/intro_to_dojo_io.html">helps with bookmarks</a>. If you read the Dojo page, you'll find that Alex Russell is wisely letting you know that there's deep voodoo going on.</p>

<p>Repeating what I said in the last section: unless Ajax is adding serious value, don't let the voodoo creep into your software and don't let your users be confused because their browsers aren't behaving like they normally do.</p>

<h2 id="thats_not_how_that_control_works">That's not how that control works!</h2>

<p>Google Suggest-style autocompletion is an obvious win. People are used to that kind of interaction. Quicken is used by millions of people. If Quicken does it, you're probably safe doing it, too. There are lots of other programs that use this kind of autocompletion, so you're really safe with that one.</p>

<p>Just because you can spruce up a text field with autocompletion doesn't mean that all widgets are ripe for Ajaxification. The <a href="http://en.wikipedia.org/wiki/Principle_of_least_astonishment">principle of least surprise</a> applies here: people have expectations of what a widget is supposed to do. Over time, those expectations can change. Once upon a time, a select box was just for selecting an item from a list. Now, select boxes are generally accepted as a way to navigate, without a button click, to another page on a site.</p>

<p>Shifts like that don't happen overnight and, as simple as it may seem, users get confused by it until they are used to it. Maybe, by now, users are so used to the lack of interface standards on the web that they're willing to accept anything.</p>

<p>If you're trying to reach a broad or non-technical audience, your best bet is to leave things as people are used to them.</p>

<h2 id="bad_control">Evil Voting Example</h2>

<img src="/static/images/ButterflyVote.png" style="float:left"/>

<p>Here's an example of a misused control: a confusing voting system in which the radio button control automatically submits the vote without any confirmation. This is not normal behavior for a radio button. Even worse, this particular example does not let you change your vote after you've set it! This is a combination of unexpected behavior <em>and</em> exceedingly unfriendly behavior.</p>

<p>The code for this example is very simple. We put a call to a function called <code>cast_vote</code> as the onclick event for the radio buttons.</p>

<div py:replace="XML(str(snippets.votehtml))"/>

<p>The JavaScript that's called sends the vote along to the server. When the response comes back, <code>voteresponse</code> will highlight the user's vote if they haven't already voted. If they <em>have</em> voted, it will switch the radio button back to the one they voted for and display that crazy error message.</p>

<div py:replace="XML(str(snippets.votejs))"/>

<h1 id="conclusions" class="exampleheader">Conclusions</h1>

<h2 id="know_your_audience">Know your audience</h2>

<p>This is what it all boils down to. You need to consider who your users are: what is their level of technical expertise? How much do they like to learn and do new things on the computer? Can you train them?</p>

<p>Those kinds of questions will narrow down the sorts of things that you can get away with. But, just because you can get away with something doesn't mean that you should do it.</p>

<p>You should use Ajax in a way that enhances the user experience: allowing users to work faster, do more with less clicks, need to learn less or think less about the actions required.</p>

<p>Build on what your users have experienced previously and avoid breaking what they already know, and you'll be able to use Ajax fearlessly to make the world a better place. Or, at least, to give your app a better face.</p>

<h1 id="further_reading" class="exampleheader">Further Reading</h1>

<p><a href="http://developer.yahoo.net/ypatterns/">Yahoo! Design Pattern Library</a> by Yahoos!</p>

<p><a href="http://sourcelabs.com/ajb/archives/2005/05/ajax_mistakes.html">Ajax Mistakes</a> by Alex Bosworth.</p>

<p><a href="http://www.baekdal.com/articles/Usability/XMLHttpRequest-guidelines">XMLHttpRequest Usability Guidelines</a> by Thomas Baekdal.</p>

<p><a href="http://www.baekdal.com/articles/Usability/usable-XMLHttpRequest/">Usable XMLHttpRequest in Practice</a> by Thomas Baekdal.</p>

<p><a href="http://www.softwareas.com/ajax-patterns">Design Patterns for Ajax Usability</a> by Michael Mahemoff</p>
</body>
</html>