from turbogears import widgets
import turbogears
from turbogears import controllers
from turbogears import validators
from turbogears import testutil
import cherrypy
from datetime import datetime

class MyFormFields(widgets.WidgetsList):
    #XXX: Since allow_extra_fields should be removed from validators.Schema we
    #      Need a validator for every input-expecting widget
    name = widgets.TextField(validator=validators.String())
    age = widgets.TextField(validator=validators.Int(), default=0)
    date = widgets.CalendarDatePicker(validator=validators.DateConverter(
                                            if_empty=datetime.now()))

myform = widgets.TableForm(fields=MyFormFields())
                           
class MyRoot(controllers.RootController):

    def index(self):
        return dict(form=myform)
    index = turbogears.expose(html="turbogears.tests.form")(index)

    def usemochi(self):
        return dict(mochi=turbogears.mochikit, form=myform)
    usemochi = turbogears.expose(html="turbogears.tests.form")(usemochi)

    def testform(self, name, date, age, tg_errors=None):
        if tg_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date
    testform = turbogears.validate(form=myform)(testform)
    testform = turbogears.expose(html="turbogears.tests.othertemplate")(
                                 testform)

    def testform_new_style(self, name, date, age):
        if cherrypy.request.validation_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date
    testform_new_style = turbogears.validate(form=myform)(testform_new_style)
    testform_new_style = turbogears.expose()(testform_new_style)


def test_form_translation():
    "Form input is translated into properly converted parameters"
    root = MyRoot()
    cherrypy.root = root
    testutil.createRequest("/testform?name=ed&date=11/05/2005&age=5")
    assert root.name == "ed"
    print root.age
    assert root.age == 5

def test_form_translation_new_style():
    "Form input is translated into properly converted parameters"
    root = MyRoot()
    cherrypy.root = root
    testutil.createRequest("/testform_new_style?name=ed&date=11/05/2005&age=5&")
    assert root.name == "ed"
    print root.age
    assert root.age == 5

def test_invalid_form_with_error_handling():
    "Invalid forms can be handled by the method"
    root = cherrypy.root
    testutil.createRequest("/testform?name=ed&age=edalso&date=11/05/2005")
    assert root.has_errors

def test_css_should_appear():
    "CSS should appear when asked for"
    root = cherrypy.root
    testutil.createRequest("/")
    print cherrypy.response.body[0]
    assert "calendar-system.css" in cherrypy.response.body[0]

def test_javascript_should_appear():
    "JavaScript should appear when asked for"
    root = cherrypy.root
    testutil.createRequest("/")
    print cherrypy.response.body[0]
    assert "calendar.js" in cherrypy.response.body[0]

def test_include_mochikit():
    "JSLinks (and MochiKit especially) can be included easily"
    root = cherrypy.root
    testutil.createRequest("/usemochi")
    print cherrypy.response.body[0]
    assert "MochiKit.js" in cherrypy.response.body[0]

def test_mochikit_everywhere():
    "MochiKit can be included everywhere by setting tg.mochikit_all"
    root = cherrypy.root
    turbogears.config.update({"global":{"tg.mochikit_all" : True}})
    testutil.createRequest("/")
    turbogears.config.update({"global":{"tg.mochikit_all" : False}})
    print cherrypy.response.body[0]
    assert "MochiKit.js" in cherrypy.response.body[0]

