import inspect
import logging
import unittest
import cStringIO as StringIO
import Cookie

import cherrypy
import sqlobject
from sqlobject.inheritance import InheritableSQLObject
from cherrypy import _cphttptools

from turbogears import database, controllers, startup, validators, config
from turbogears.util import get_model

config.update({"global" : {"tg.new_style_logging" : True}})
config.update({"global" : {"autoreload.on" : False}})
cherrypy.server.start(serverClass=None, initOnly=True)
database.set_db_uri("sqlite:///:memory:")

def create_request(request, method="GET", protocol="HTTP/1.1",
    headers={}, rfile=None, clientAddress="127.0.0.1",
    remoteHost="localhost", scheme="http"):
    if not rfile:
        rfile = StringIO.StringIO("")
    headerList = [(key, value) for key, value in headers.items()]
    headerList.append(("Host", "localhost"))
    if not hasattr(cherrypy.root, "started"):
        startup.startTurboGears()
        cherrypy.root.started = True
    req = _cphttptools.Request(clientAddress, 80, remoteHost, scheme)
    cherrypy.serving.request = req
    cherrypy.serving.response = _cphttptools.Response()
    req.run(" ".join((method, request, protocol)), headerList, rfile)

createRequest = create_request

def _return_directly(output, *args):
    return output

class DummyRequest:
    "A very simple dummy request"
    def __init__(self):
        self.headers = {}
    def purge__(self):
        pass

def call(method, *args, **kw):
    orig_proc_output = controllers._process_output
    controllers._process_output = _return_directly
    cherrypy.serving.response = _cphttptools.Response()
    cherrypy.serving.request = DummyRequest()
    try:
        return method(*args, **kw)
    finally:
        del cherrypy.serving.request
        controllers._process_output = orig_proc_output


class DBTest(unittest.TestCase):
    model = None

    def setUp(self):
        if not self.model:
            self.model = get_model()
            if not self.model:
                raise "Unable to run database tests without a model"

        for item in self.model.__dict__.values():
            if inspect.isclass(item) and issubclass(item,
                sqlobject.SQLObject) and item != sqlobject.SQLObject \
                and item != InheritableSQLObject:
                item.createTable(ifNotExists=True)

    def tearDown(self):
        database.rollback_all()

def reset_cp():
    cherrypy.root = None

def catch_validation_errors(widget, value):     
    """ Catches and unpacks validation errors. For testing purposes. """
    errors = {}
    try:
        value = widget.validate(value)
    except validators.Invalid, e:
        if hasattr(e, 'unpack_errors'):
            errors = e.unpack_errors()
        else:
            errors = e
    return value, errors

class MemoryListHandler(logging.Handler):
    
    def __init__(self):
        logging.Handler.__init__(self, level=logging.DEBUG)
        self.log = []
    
    def emit(self, record):
        print "Got record: %s" % record
        print "formatted as: %s" % self.format(record)
        self.log.append(self.format(record))
    
    def print_log(self):
        print "\n".join(self.log)
        self.log = []
    
    def get_log(self):
        log = self.log
        self.log = []
        return log

_memhandler = MemoryListHandler()
_currentcat = None

def capture_log(category):
    """Category can either be a single category (a string like 'foo.bar')
    or a list of them. You <em>must</em> call print_log() to reset when
    you're done."""
    global _currentcat
    assert not _currentcat
    if not isinstance(category, list) and not isinstance(category, tuple):
        category = [category]
    _currentcat = category
    for cat in category:
        log = logging.getLogger(cat)
        log.setLevel(logging.DEBUG)
        log.addHandler(_memhandler)

def _reset_logging():
    """Manages the resetting of the loggers"""
    global _currentcat
    if not _currentcat:
        return
    for cat in _currentcat:
        log = logging.getLogger(cat)
        log.removeHandler(_memhandler)
    _currentcat = None

def print_log():
    """Prints the log captured by capture_log to stdout, resets that log
    and resets the temporarily added handlers."""
    _reset_logging()
    _memhandler.print_log()

def get_log():
    """Returns the list of log messages captured by capture_log,
    resets that log and resets the temporarily added handlers."""
    _reset_logging()
    return _memhandler.get_log()

__all__ = ["create_request", "call", "DBTest", "createRequest",
    "capture_log", "print_log", "get_log"]
