import cherrypy
import sha
import md5
import random
import turbogears
from datetime import *
from sqlalchemy import *
from sqlalchemy.ext.activemapper import *
from turbogears.visit.api import BaseVisitManager, Visit

import logging
log = logging.getLogger("turbogears.identity.savisit")

__engine__= turbogears.database.PackageEngine( 'turbogears.visit' )

class SqlAlchemyVisitManager(BaseVisitManager):
    def __init__(self, timeout):
        super(SqlAlchemyVisitManager,self).__init__( timeout )
        
    def create_model(self):
        try:
            TG_Visit.table.create()
            objectstore.commit()
        except SQLError:
            pass
            
    def new_visit_with_key(self, visit_key):
        visit= TG_Visit( visit_key=visit_key, expiry=datetime.now()+self.timeout )
        visit.commit()
        return Visit( visit_key, True )
        
    def visit_for_key(self, visit_key):
        '''
        Return the visit for this key or None if the visit doesn't exist or has
        expired.
        '''
        visit= TG_Visit.lookup_visit( visit_key )
        now= datetime.now()
        if not visit or visit.expiry < now:
            return None
        # Visit hasn't expired, extend it
        self.update_visit( visit_key, now+self.timeout )
        return Visit( visit_key, False )
        
    def update_queued_visits(self, queue):
        try:
            table= TG_Visit.table
            # Now update each of the visits with the most recent expiry
            for visit_key,expiry in queue.items():
                log.info( "updating visit (%s) to expire at %s", visit_key,
                          expiry )
                table.update( table.c.visit_key==visit_key,
                                  values={'expiry': expiry} ).execute()
            objectstore.commit()
        except:
            objectstore.clear()
            raise
            

class TG_Visit(ActiveMapper):
    class mapping:
        visit_key= column( String(40), primary_key=True )
        created= column( DateTime, default=datetime.now )
        expiry= column( DateTime )

    def lookup_visit( cls, visit_key ):
        return TG_Visit.get( visit_key );
    lookup_visit= classmethod(lookup_visit)
