from turbogears.visit.api import BaseVisitManager, Visit

from sqlobject import *
from sqlobject.sqlbuilder import *
from turbogears.database import PackageHub
from datetime import timedelta, datetime

hub = PackageHub("turbogears.visit")
__connection__ = hub


class SqlObjectVisitManager(BaseVisitManager):
    def __init__(self, timeout):
        super(SqlObjectVisitManager,self).__init__( timeout )

    def create_model(self):
        try:
            # Create the Visit table if it doesn't already exist (idea from CatWalk)
            hub.begin()
            TG_Visit.createTable(ifNotExists=True)
            hub.commit()
            hub.end()
        except KeyError:
            # No database configured...
            log.info( "No database is configured: Visit Tracking is disabled." )
            return

    def new_visit_with_key(self, visit_key):
        hub.begin()
        visit= TG_Visit( visit_key=visit_key, expiry=datetime.now()+self.timeout )
        hub.commit()
        hub.end()
        return Visit( visit_key, True )

    def visit_for_key(self, visit_key):
        '''
        Return the visit for this key or None if the visit doesn't exist or has
        expired.
        '''
        visit= TG_Visit.lookup_visit( visit_key )
        now= datetime.now()
        if not visit or visit.expiry < now:
            return None
        # Visit hasn't expired, extend it
        self.update_visit( visit_key, now+self.timeout )
        return Visit( visit_key, False )

    def update_queued_visits(self, queue):
        try:
            hub.begin()
            conn= hub.getConnection()
            # Now update each of the visits with the most recent expiry
            for visit_key,expiry in queue.items():
                u= Update( TG_Visit.q, {TG_Visit.q.expiry.fieldName:expiry},
                           where=(TG_Visit.q.visit_key==visit_key) )
                conn.query( conn.sqlrepr(u) )
            hub.commit()
            hub.end()
        except:
            hub.rollback()
            hub.end()
            raise


class TG_Visit(SQLObject):
    class sqlmeta:
        table="tg_visit"

    visit_key= StringCol( length=40, alternateID=True,
                          alternateMethodName="by_visit_key" )
    created= DateTimeCol( default=datetime.now )
    expiry= DateTimeCol()

    def lookup_visit( cls, visit_key ):
        try:
            return cls.by_visit_key( visit_key )
        except SQLObjectNotFound:
            return None
    lookup_visit= classmethod(lookup_visit)
