import os
from cherrypy import request
from turbogears import startup, config
from turbogears.util import parse_http_accept_header
from turbogears.i18n.utils import lang_in_gettext_format
from turbogears.widgets.base import JSLink, CoreWD, RenderOnlyWD, static

__all__ = ["LocalizableJSLink"]

class LocalizableJSLink(JSLink):
    def update_params(self, d):
        accept_language = parse_http_accept_header(            
            request.headers.get("Accept-Language")
        ) or ['']
        accept_charset = parse_http_accept_header(
            request.headers.get("Accept-Charset")
        ) or ['']
        base_dir = config.get("static_filter.dir", 
                              path="/tg_widgets/%s" % self.mod)
        def find_link():
            for name_pattern in self.name:
                for lang in accept_language:
                    for charset in accept_charset:
                        params = dict()
                        params["lang"] = lang
                        params["charset"] = charset
                        params["gettext_lang"] = lang_in_gettext_format(lang)
                        params["gettext_charset"] = charset.upper()
                        params["custom_lang"] = self.custom_lang(lang)
                        params["custom_charset"] = self.custom_charset(charset)
                        name = name_pattern % params
                        if os.path.exists(base_dir + '/' + name):
                            link = "/%stg_widgets/%s/%s" % (startup.webpath,
                                                            self.mod,
                                                            name)
                            return link
        d["link"] = find_link()

    def custom_lang(self, lang):
        return None

    def custom_charset(self, charset):
        return None

class LocalizableJSLinkDesc(CoreWD, RenderOnlyWD):
    name = "Localizable JS Link"
    default_lang = "en"
    for_widget = LocalizableJSLink(static, 
                                   ["calendar/lang/calendar-%(lang)s-%(charset)s.js",
                                    "calendar/lang/calendar-%(lang)s.js",
                                    "calendar/lang/calendar-%s.js" % default_lang])
