from turbogears.widgets.base import CSSLink, JSLink, CSSSource, JSSource, \
                                    Widget, CoreWD, static, js_location
class Tabber(Widget):
    """This widget includes the tabber js and css into your rendered
    page so you can create tabbed divs by assigning them the 'tabber' 
    and 'tabbertab' classes.
    """
    css = [CSSLink(static,"tabber/tabber.css", media="screen")]
    javascript = [JSLink(static, "tabber/tabber-minimized.js"),
          JSSource("""
    /* Optional: Temporarily hide the "tabber" class so it does not "flash"
       on the page as plain HTML. After tabber runs, the class is changed
       to "tabberlive" and it will appear. */
    document.write('<style type="text/css">.tabber{display:none;}<\/style>');
    """),
    ]

class TabberDesc(CoreWD):
    name = "Tabber"
    for_widget = Tabber()
    template = """<div class="tabber">
        <div class="tabbertab"><h2>Tab 1</h2></div>
        <div class="tabbertab"><h2>Tab 2</h2></div>
        <div class="tabbertab"><h2>Tab 3</h2></div>
        </div>"""

class SyntaxHighlighter(Widget):
    """This widget includes the syntax highlighter js and css into your 
    rendered page to syntax-hightlight textareas named 'code'. The supported
    languages can be listed at the 'languages' __init__ parameter.
    """
    _lang_mapper = {
        'C' : 'shBrushCSharp.js',
        'Delphi' : 'shBrushDelphi.js',
        'JScript' : 'shBrushJScript.js',
        'Php' : 'shBrushPhp.js',
        'Python' : 'shBrushPython.js',
        'Sql' : 'shBrushSql.js',
        'Vb' : 'shBrushVb.js',
        'Xml' : 'shBrushXml.js',
    }
    css = [CSSLink(static,"sh/SyntaxHighlighter.css")]

    def __init__(self, languages=['Python', 'Xml']):
        javascript = [JSLink(static, 'sh/shCore.js', 
                             location=js_location.bodybottom)]
        for lang in languages:
            try:
                source = self._lang_mapper[lang]
                javascript.append(JSLink(static, 'sh/' + source, 
                                         location=js_location.bodybottom))
            except KeyError:
                raise ValueError, ("Unsupported language %s. Available "
                                   "languages: '%s'" % 
                                   (lang, ', '.join(self._lang_mapper.keys())))
        javascript.append(JSSource("dp.SyntaxHighlighter.HighlightAll('code');",
                                   location=js_location.bodybottom))
        self.javascript = javascript 

class SyntaxHighlighterDesc(CoreWD):
    name = "Syntax Highlighter"
    for_widget = SyntaxHighlighter()
    template = """\
    <textarea name="code" class="py">
        def say_hello():
            print "Hello world!"
    </textarea>"""

