import cherrypy
import turbogears
from turbogears import controllers, validators
from turbogears import identity
from model import Page
from docutils.core import publish_parts
import re
from sqlobject import SQLObjectNotFound

wikiwords = re.compile(r"\b([A-Z]\w+[A-Z]+\w+)")

class Root(controllers.RootController):
    @turbogears.expose(html=".templates.page")
    def default(self, pagename):
        return self.index(pagename)
        
    @turbogears.expose(html=".templates.page")
    def index(self, pagename="FrontPage"):
        try:
            page = Page.byPagename(pagename)
        except SQLObjectNotFound:
            raise turbogears.redirect("/notfound", pagename= pagename)
        content = publish_parts(page.data, writer_name="html")["html_body"]
        root = str(turbogears.url("/"))
        content = wikiwords.sub(r'<a href="%s\1">\1</a>' % root, content)
        return dict(data=content, pagename=page.pagename)
        
    @turbogears.expose(html=".templates.edit")
    def edit(self, pagename):
        page = Page.byPagename(pagename)
        return dict(pagename=page.pagename, data=page.data, new=False)
        
    @turbogears.expose()
    @turbogears.validate(validators=dict(new=validators.StringBoolean()))
    def save(self, pagename, data, submit, new):
        if new:
            page = Page(pagename=pagename, data=data)
        else:
            page = Page.byPagename(pagename)
            page.data = data
        turbogears.flash("Changes saved!")
        raise turbogears.redirect("/%s" % pagename)
        
    @turbogears.expose(html=".templates.edit")
    def notfound(self, pagename):
        return dict(pagename=pagename, data="", new=True)
        
    @turbogears.expose(html=".templates.pagelist", allow_json=True)
    def pagelist(self):
        pages = [page.pagename for page in Page.select(orderBy=Page.q.pagename)]
        return dict(pages=pages)

    @turbogears.expose( html=".templates.login" )
    def login( self, *args, **kw ):
        if hasattr(cherrypy.request,"identity_errors"):
            msg= _("You must provide your credentials before accessing this resource.")
            previous_url= cherrypy.request.path
        else:
            msg= _("Please log in.")
            previous_url= cherrypy.request.headerMap.get( "Referer", "/" )
        cherrypy.response.status=403
        return dict( message=msg, previous_url=previous_url, logging_in=True,
                     original_parameters=cherrypy.request.paramMap )

    @turbogears.expose()
    def logout( self ):
        identity.current.logout()
        raise cherrypy.HTTPRedirect(turbogears.url("/"))
