import cherrypy
import turbogears
from turbogears import controllers, expose, redirect
from turbogears import identity
from turbogears.toolbox.catwalk import CatWalk
import model

class Users:
    @expose(template="tutorial.templates.users")
    def index(self):
        users = model.User.select()
        return dict(users=users)

    @expose(template="tutorial.templates.user")
    def default(self, userID):
        try:
            userID = int(userID)
            user = model.User.get(userID)
        except (ValueError, model.SQLObjectNotFound):
            raise cherrypy.NotFound
        else:
            return dict(user=user)

    @expose()
    def add(self, email):
        # Remove extra spaces
        email = email.strip()
        # Remove null bytes (they can seriously screw up the database)
        email = email.replace('\x00', '')
        if email:
            try:
                user = model.User.byEmail(email)
            except model.SQLObjectNotFound:
                user = model.User(email=email)
                turbogears.flash("User %s added!" % email)
            else:
                turbogears.flash("User %s already exists!" % email)
        else:
            turbogears.flash("E-mail must be non-empty!")
        redirect('index')

    @expose()
    def remove(self, email):
        try:
            user = model.User.byEmail(email)
        except model.SQLObjectNotFound:
            turbogears.flash("The user %s does not exist. (Did someone else remove it?)" % email)
        else:
            for userList in user.lists:
                for item in userList.items:
                    item.destroySelf()
                userList.destroySelf()
            user.destroySelf()
            turbogears.flash("User %s removed!" % email)
        redirect('index')

class Root(controllers.RootController):
    users = Users()
    catwalk = CatWalk(model)

    @expose(template="tutorial.templates.welcome")
    def index(self):
        return dict()

    @expose(template="tutorial.templates.about")
    def about(self, author="Brian Beck"):
        return dict(author=author)

    @expose()
    def addList(self, userID, title):
        try:
            userID = int(userID)
            user = model.User.get(userID)
        except (ValueError, model.SQLObjectNotFound):
            raise cherrypy.NotFound
        else:
            # Remove extra spaces
            title = title.strip()
            # Remove null bytes (they can seriously screw up the database)
            title = title.replace('\x00', '')
            if title:
                theList = model.List(title=title, user=user)
                turbogears.flash("List created!")
            else:
                turbogears.flash("Title must be non-empty!")
        redirect('/users/%s' % userID)

    @expose()
    def removeList(self, userID, listID):
        try:
            listID = int(listID)
            theList = model.List.get(listID)
        except ValueError:
            turbogears.flash("Invalid list!")
        except model.SQLObjectNotFound:
            turbogears.flash("List not found! (Did someone else remove it?)")
        else:
            for item in theList.items:
                item.destroySelf()
            theList.destroySelf()
            turbogears.flash("List deleted!")
        redirect('/users/%s' % userID)

    @expose()
    def addItem(self, userID, listID, value):
        try:
            listID = int(listID)
            theList = model.List.get(listID)
        except ValueError:
            turbogears.flash("Invalid list!")
        except model.SQLObjectNotFound:
            turbogears.flash("List not found! (Did someone else remove it?)")
        else:
            # Remove extra spaces
            value = value.strip()
            # Remove null bytes (they can seriously screw up the database)
            value = value.replace('\x00', '')
            if value:
                item = model.Item(listID=listID, value=value)
                turbogears.flash("Item added!")
            else:
                turbogears.flash("Item must be non-empty!")
        redirect('/users/%s' % userID)

    @expose()
    def removeItem(self, userID, itemID):
        try:
            itemID = int(itemID)
            item = model.Item.get(itemID)
        except ValueError:
            turbogears.flash("Invalid item!")
        except model.SQLObjectNotFound:
            turbogears.flash("No such item! (Did someone else remove it?)")
        else:
            item.destroySelf()
            turbogears.flash("Item removed!")
        redirect('/users/%s' % userID)

    @expose(format="json")
    def editItem(self, itemID, value):
        try:
            itemID = int(itemID)
            item = model.Item.get(itemID)
        except (ValueError, model.SQLObjectNotFound):
            raise cherrypy.NotFound
        else:
            # Remove extra spaces
            value = value.strip()
            # Remove null bytes (they can seriously screw up the database)
            value = value.replace('\x00', '')
            if value:
                item.value = value
            else:
                value = item.value
        return dict(value=value)